import { Rule, SubjectType } from './types';
import { Ability } from './ability';

export class AbilityBuilder {
    private rules: Rule[] = [];

    can(actions: string | string[], subjects: SubjectType | SubjectType[], conditions?: Record<string, any>): this {
        this._addRule(actions, subjects, conditions, false);
        return this;
    }

    cannot(actions: string | string[], subjects: SubjectType | SubjectType[], conditions?: Record<string, any>): this {
        this._addRule(actions, subjects, conditions, true);
        return this;
    }

    build(): Ability {
        return new Ability(this.rules);
    }

    private _addRule(
        actions: string | string[],
        subjects: SubjectType | SubjectType[],
        conditions: Record<string, any> | undefined,
        inverted: boolean
    ) {
        const actionsArr = Array.isArray(actions) ? actions : [actions];
        const subjectsArr = Array.isArray(subjects) ? subjects : [subjects];
        this.rules.push({ actions: actionsArr, subjects: subjectsArr, conditions, inverted });
    }
}
